#' ExtractMotifs is a short function designed to help in processign MERMADE results
#'
#' @param mermadeResults Path to the MERMADE ".html"output file to be processed
#' @return Three files in the current active directory containing the either the shorter unique, the longer unique or all the Unique motifs (NOTE: if a motif is unique will appear in both)
#' @export
#' @examples
#' x <-ExtractMotifs("<path_to_html_file>","<Ratio_Threshold>","<foreground>","<path_to_Barcode_List>")


ExtractMotifs <- function(mermadeResults,ratio_thresh=0,foreground=0, BClist) {

  #execute
  htmlres <- htmltab::htmltab(mermadeResults,which = 1, header = 1, rm_nodata_cols = F)
  y<-1;
  while (y<1000) {
    try({
    y = y+1;
    htmlres2 <- htmltab::htmltab(mermadeResults,which = y,header = 1, rm_nodata_cols = F)
    htmlres <- rbind(htmlres,htmlres2)
    }, silent =TRUE)
  }
  #BClist2 <- as.vector(read.delim(BClist,header = FALSE, sep = ","))
  # reading the sequences
  seqs1 <- htmlres[,-(c(5,7))]
  # seqs1<- data.frame(htmlres$Seed,stringsAsFactors = FALSE)
  # we need to include the first column of the table
  #seqs1$barcode <-html$BC
  # filter by enrichment
  seqs1$enrichment <- as.numeric(htmlres$Ratio)
  seqs1$Ratio <- as.numeric(as.character(seqs1$Ratio))
  seqs1$Foreground <- as.numeric(as.character(seqs1$Foreground))
  seqs1 <- seqs1[seqs1$Ratio >=ratio_thresh, ]
  seqs1 <- seqs1[seqs1$Foreground >=foreground, ]
  seqs1unique <- unique(seqs1[,-(c(4:7))])
  nseqs <- length(seqs1unique$Seed)

  seq2 <-seqs1unique
  seq2a <- seq2[-(1:nseqs),]
  seq3a<- seq2a

  y<-0;
  while (y<nseqs) {
    y = y+1;

    seq2$query <- seqs1unique$Seed[y]
    seq2$BCq <- seqs1unique$BC[y]
    seq2$query_length <- nchar(seqs1unique$Seed[y])
    seq2$BCQuery <- paste(seq2$query, seq2$BCq, sep = ".")
    seq2$detect <-stringr::str_detect(seqs1unique$Seed,seqs1unique$Seed[y])
    seq2$count <- stringr::str_count(seqs1unique$Seed,seqs1unique$Seed[y])
    seq2a <- seq2[seq2$detect=="TRUE",]
    seq3a <- rbind(seq3a,seq2a)
  }
#' clan up
#' rm(list= c(htmlres, htmlres2,seq2, seq2a, nseqs, seqs1))
#'
  seq3a$SizeDifference <- ((as.numeric(seq3a$K))-seq3a$query_length)

  #' To calculate unique motifs regardless of the barcode

      seq3cnobc             <- seq3a[(seq3a$Seed!=seq3a$query),] # these are the short ones contained in longer ones
      seq3dnobc             <- seq3a[(seq3a$Seed==seq3a$query),] # These are all the unique motifs selected
      Unfilt_NOBC_Long_All  <- seq3dnobc[!(seq3dnobc$Seed %in% seq3cnobc$query),]  #' long sequences regardless of barcode matiching
      Unfilt_NOBC_Motifs_summary <- unique(Unfilt_NOBC_Long_All[,3])        #' Contains Motifs that can be found in other longer ones becuase it comes from a barcode matched search

      write.csv(Unfilt_NOBC_Motifs_summary, "./NO_BC_Summary_of_long_Motifs.csv") #' Contains the long unique Motifs

  #' Matches Motifs with Barcodes
  seq3c                <- seq3a[(seq3a$Seed!=seq3a$query & seq3a$BC==seq3a$BCq),] # these are the short ones contained in longer ones
  seq3d                <- seq3a[(seq3a$Seed==seq3a$query & seq3a$BC==seq3a$BCq),] # These are all the unique motifs selected
  Res_Motifs_in_BCs    <- as.data.frame(plyr::count(seq3d,"Seed")) # Motifs found in different BCs
  Res_Motifs_x_BC      <- as.data.frame(plyr::count(seq3d,"BC")) # Number of motifs found for each BC

  Unfilt_Long_All  <- seq3d[!(seq3d$Seed %in% seq3c$query),]  #' This contains all the long sequences with the same Barcodes
  Unfilt_Short_All <- seq3d[(seq3d$Seed %in% seq3c$query),]   #' This contains all the short sequences contained in others with the same barcodes
  Unfilt_Motifs_summary <- unique(Unfilt_Long_All[,3])        #' Contains Motifs that can be found in other longer ones becuase it comes from a barcode matched search

  write.csv(Unfilt_Motifs_summary, "./Summary_of_long_Motifs.csv") #' Contains the long unique Motifs

  if (BClist!=""){
                  BClist2 <- as.vector(read.delim(BClist,header = FALSE, sep = ","))
                  BC_filt <- Unfilt_Long_All[(Unfilt_Long_All$BC %in% BClist2$V1),]
                  BC_filt_freq <- as.data.frame(plyr::count(BC_filt,"Seed"))

                   #' we apply a second pass to reanalyse the sequences that were initially selected
                   #' because they were mtached by barcode. In this second stage we look for new matches only
                   #' among the selected sequences
                        y<-0;
                        nseqsfil <- length(BC_filt$Seed)
                        seqsfil <- BC_filt
                        seqsfila <- seqsfil[-(1:nseqs),]
                        seqsfil3a<- seqsfila
                        while (y<nseqsfil) {
                                            y = y+1;
                                            seqsfil$query <- BC_filt$Seed[y]
                                            seqsfil$BCq <- BC_filt$BC[y]
                                            seqsfil$query_length <- nchar(BC_filt$Seed[y])
                                            seqsfil$BCQuery <- paste(seqsfil$query, seqsfil$BCq, sep = ".")
                                            seqsfil$detect <-stringr::str_detect(BC_filt$Seed,BC_filt$Seed[y])
                                            seqsfil$count <- stringr::str_count(BC_filt$Seed,BC_filt$Seed[y])
                                            seqsfila <- seqsfil[seqsfil$detect=="TRUE",]
                                            seqsfil3a <- rbind(seqsfil3a,seqsfila)
                                          }
                        seqsfil3a$SizeDifference <- ((as.numeric(seqsfil3a$K))-seqsfil3a$query_length)
                        seqsfil3c <- seqsfil3a[(seqsfil3a$Seed!=seqsfil3a$query),] # these are the short ones contained in longer ones
                        seqsfil3d <- seqsfil3a[(seqsfil3a$Seed==seqsfil3a$query),] # These are all the unique motifs selected

                        BC_Filt_Long  <- seqsfil3d[!(seqsfil3d$Seed %in% seqsfil3c$query),]   #' this contains all the long motifs for selected Barcodes
                        BC_Filt_Short <- seqsfil3d[(seqsfil3d$Seed %in% seqsfil3c$query),]    #' this contains all the short motifs for removed for the selected Barcodes
                        Res_Filt_Motifs      <- as.data.frame(plyr::count(BC_Filt_Long,"Seed"))

                        write.csv(Res_Filt_Motifs[1],"./BC_selected_Longest_Seqs.csv")        # writes the longest Motifs with the preferred barcodes in the current wroking directory

                        DNASeqs <- Biostrings::DNAStringSet(x=Res_Filt_Motifs$Seed)
                        testalign <- DECIPHER::AlignSeqs(DNASeqs)
                        DECIPHER::BrowseSeqs(testalign)

                    #' cleanup
                    # rm(BClist2,BC_filt,BC_filt_freq,y,nseqsfil,seqsfil,seqsfila, DNASeqs, testalign,seqsfil3a,seqsfil3c,seqsfil3d)

                        } #' finishes the barcode selection analysis

  #' clean up
  # rm(htmlres, htmlres2,seq2, seq2a, nseqs, seqs1, seq3a,seq3c,seq3cnobc,seq3d,seq3dnobc)

}       #' finishes the funtion

